import 'package:align_positioned/align_positioned.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/scheduler.dart';
import 'package:syncfusion_flutter_charts/charts.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

void main() {
  runApp(ProviderScope(child: MyApp()));
}

@immutable
class ExternalTrackballState {
  final String header;
  final String trackballName;
  const ExternalTrackballState(this.header, this.trackballName);

  @override
  bool operator ==(Object other) =>
      other is ExternalTrackballState &&
      other.header == header &&
      other.trackballName == trackballName;

  @override
  int get hashCode => header.hashCode ^ trackballName.hashCode;
}

class TrackballHeaderWidget extends ConsumerWidget {
  static final stateProvider =
      StateProvider((ref) => const ExternalTrackballState("", ""));

  const TrackballHeaderWidget({super.key});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final state = ref.watch(stateProvider);
    return ClipRRect(
      borderRadius: BorderRadius.circular(15),
      child: Container(
        height: 80,
        width: 300,
        color: Colors.red.shade100,
        child: Column(
          children: [
            Text(state.header, textAlign: TextAlign.center),
            const SizedBox(
              height: 3,
              width: 85,
              child: ColoredBox(color: Colors.red),
            ),
            Text(state.trackballName),
          ],
        ),
      ),
    );
  }
}

class MyApp extends ConsumerWidget {
  static final xPositionProvider = StateProvider((ref) => 0.0);

  MyApp({super.key});

  late TrackballBehavior trackballBehavior;

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final xPosition = ref.watch(xPositionProvider);
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Syncfusion Flutter chart'),
        ),
        body: AlignPositioned.relative(
          child: const TrackballHeaderWidget(),
          alignment: Alignment.topLeft,
          dx: xPosition + 12,
          dy: 15.0,
          moveByChildWidth: -0.5,
          container: SizedBox(
              height: 300,
              width: 900,
              child: MouseRegion(
                hitTestBehavior: HitTestBehavior.translucent,
                onExit: (event) {
                  // you can clear your custom trackball widget here
                  print('Trackball hides on exit.');
                },
                child: SfCartesianChart(
                  // margin: const EdgeInsets.all(0),
                  primaryXAxis: CategoryAxis(),
                  onTrackballPositionChanging: (trackballArgs) {
                    final x = trackballArgs.chartPointInfo.xPosition;
                    if (x != null) {
                      ref.read(xPositionProvider.state).state = x;
                    }
                  },
                  trackballBehavior: TrackballBehavior(
                      builder: (context, trackballDetails) {
                        String header = '';
                        String trackballName = '';
                        int index = 0;
                        TrackballGroupingModeInfo? groupingModeInfo =
                            trackballDetails.groupingModeInfo;
                        if (groupingModeInfo != null) {
                          for (var element in groupingModeInfo.points) {
                            header = element.x.toString();
                            String seriesName = groupingModeInfo
                                    .visibleSeriesList[index].name ??
                                "Series${groupingModeInfo.visibleSeriesIndices[index]}";
                            String seriesText = "$seriesName: ${element.y}\n";
                            trackballName += seriesText;
                            index++;
                          }
                        }
                        SchedulerBinding.instance.addPostFrameCallback((_) {
                          ref
                                  .read(TrackballHeaderWidget.stateProvider.state)
                                  .state =
                              ExternalTrackballState(header, trackballName);
                        });
                        return const SizedBox.shrink();
                      },
                      activationMode: ActivationMode.singleTap,
                      enable: true,
                      tooltipDisplayMode: TrackballDisplayMode.groupAllPoints),
                  series: <ChartSeries<ChartSampleData, String>>[
                    StackedColumnSeries<ChartSampleData, String>(
                        dataSource: chartData,
                        xValueMapper: (ChartSampleData sales, _) => sales.x,
                        yValueMapper: (ChartSampleData sales, _) => sales.y,
                        name: 'Series1'),
                    StackedColumnSeries<ChartSampleData, String>(
                      dataSource: chartData,
                      xValueMapper: (ChartSampleData sales, _) => sales.x,
                      yValueMapper: (ChartSampleData sales, _) => sales.yValue,
                      name: 'Series2',
                    )
                  ],
                ),
              )),
        ),
      ),
    );
  }
}

class ChartSampleData {
  ChartSampleData(this.x, this.y, this.yValue, this.secondSeriesYValue);
  final String? x;
  final double? y;
  final double? yValue;
  final double? secondSeriesYValue;
}

final List<ChartSampleData> chartData = <ChartSampleData>[
  ChartSampleData('Jan', 6, 6, -1),
  ChartSampleData('Feb', 8, 8, -1.5),
  ChartSampleData('Mar', 12, 11, -2),
  ChartSampleData('Apr', 15.5, 16, -2.5),
  ChartSampleData('May', 20, 21, -3),
  ChartSampleData('June', 24, 25, -3.5),
];

/*
class MyApp extends ConsumerWidget {
  const MyApp({super.key});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Syncfusion Flutter chart'),
        ),
        body: Column(
          children: <Widget>[
            const TrackballHeaderWidget(),
            SizedBox(
              height: 300,
              width: 900,
              child: SfCartesianChart(
                primaryXAxis: CategoryAxis(),
                trackballBehavior: TrackballBehavior(
                    builder: (context, trackballDetails) {
                      String header = '';
                      String trackballName = '';
                      int index = 0;
                      TrackballGroupingModeInfo? groupingModeInfo =
                          trackballDetails.groupingModeInfo;
                      if (groupingModeInfo != null) {
                        for (var element in groupingModeInfo.points) {
                          header = element.x.toString();
                          String seriesName = groupingModeInfo
                                  .visibleSeriesList[index].name ??
                              "Series${groupingModeInfo.visibleSeriesIndices[index]}";
                          String seriesText = "$seriesName: ${element.y}\n";
                          trackballName += seriesText;
                          index++;
                        }
                      }
                      SchedulerBinding.instance.addPostFrameCallback((_) {
                        ref
                                .read(TrackballHeaderWidget.stateProvider.state)
                                .state =
                            ExternalTrackballState(header, trackballName);
                      });
                      return const SizedBox.shrink();
                    },
                    activationMode: ActivationMode.singleTap,
                    enable: true,
                    tooltipDisplayMode: TrackballDisplayMode.groupAllPoints),
                series: <ChartSeries<ChartSampleData, String>>[
                  StackedColumnSeries<ChartSampleData, String>(
                      dataSource: chartData,
                      xValueMapper: (ChartSampleData sales, _) => sales.x,
                      yValueMapper: (ChartSampleData sales, _) => sales.y,
                      name: 'Series1'),
                  StackedColumnSeries<ChartSampleData, String>(
                    dataSource: chartData,
                    xValueMapper: (ChartSampleData sales, _) => sales.x,
                    yValueMapper: (ChartSampleData sales, _) => sales.yValue,
                    name: 'Series2',
                  )
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }
}
*/